package cn.com.qmth.stmms.admin.exam;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.admin.dto.ObjectiveQuestionDTO;
import cn.com.qmth.stmms.admin.dto.QuestionDTO;
import cn.com.qmth.stmms.admin.dto.SubjectQuestionDTO;
import cn.com.qmth.stmms.admin.dto.SubjectiveQuestionDTO;
import cn.com.qmth.stmms.admin.thread.ScoreCalculateThread;
import cn.com.qmth.stmms.admin.thread.ScoreReportThread;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.query.ExamQuestionSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.query.ExamSubjectSearchQuery;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.report.service.ReportService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.ObjectivePolicy;
import cn.com.qmth.stmms.common.enums.ObjectiveStatus;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.enums.SubjectiveStatus;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.ImportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;

@Controller("examPaperController")
@RequestMapping("/admin/exam/paper")
public class PaperController extends BaseExamController {

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private ExamService examService;

    @Autowired
    private FileService fileService;

    @Qualifier("task-executor")
    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @Autowired
    private LockService lockService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ReportService reportService;

    @Autowired
    private MarkService markService;

    @Logging(menu = "试卷管理查询", type = LogType.QUERY)
    @RequestMapping
    public String list(Model model, HttpServletRequest request, ExamSubjectSearchQuery query,
            @RequestParam(required = false) Boolean upload) {
        WebUser wu = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        if (upload != null) {
            if (upload.booleanValue()) {
                query.setUploadCountGt(0);
            } else {
                query.setUploadCountEqual(0);
            }
        }
        query = subjectService.findByQuery(query);
        for (ExamSubject subject : query.getResult()) {
            List<String> paperTypes = questionService.getPaperType(examId, subject.getCode());
            subject.setPaperType(StringUtils.join(paperTypes, ","));
            subject.setPaperAnswerUrl(fileService);
            subject.setLocked(lockService.isLocked(LockType.SCORE_CALCULATE, examId, subject.getCode()));
        }
        model.addAttribute("query", query);
        model.addAttribute("upload", upload);
        model.addAttribute("subjectList", getExamSubject(examId, wu));
        model.addAttribute("levelList", subjectService.listLevel(examId));
        model.addAttribute("categoryList", subjectService.listCategory(examId));
        model.addAttribute("fileServer", fileService.getFileServer());
        model.addAttribute("examLock", lockService.isLocked(LockType.SCORE_CALCULATE, examId));
        Exam exam = examService.findById(examId);
        model.addAttribute("examType", exam.getType());
        return "modules/exam/paperList";
    }

    @Logging(menu = "科目详情", type = LogType.QUERY)
    @RequestMapping("/detail")
    public String detail(Model model, HttpServletRequest request, @RequestParam String subjectCode,
            @RequestParam(required = false) Boolean objective) {
        int examId = getSessionExamId(request);
        ExamSubject subject = subjectService.find(examId, subjectCode);
        if (subject != null) {
            List<ExamQuestion> list = new LinkedList<ExamQuestion>();
            if (objective == null || objective) {
                List<ExamQuestion> objectiveList = questionService.findByExamAndSubjectAndObjective(examId,
                        subjectCode, true);
                list.addAll(objectiveList);
            }
            if (objective == null || !objective) {
                List<ExamQuestion> subjectiveList = questionService.findByExamAndSubjectAndObjective(examId,
                        subjectCode, false);
                list.addAll(subjectiveList);
            }
            model.addAttribute("questionList", list);
            model.addAttribute("subject", subject);
            model.addAttribute("objective", objective);
            return "modules/exam/paperDetail";
        } else {
            return "redirect:/admin/exam/paper";
        }
    }

    @RequestMapping(value = "/template")
    public String importTemplate(HttpServletResponse response, @RequestParam Boolean objective,
            RedirectAttributes redirectAttributes) {
        try {
            String fileName = objective ? "客观题导入模板.xlsx" : "主观题导入模板.xlsx";
            String title = objective ? "客观题数据" : "主观题数据";
            List<QuestionDTO> list = new LinkedList<QuestionDTO>();
            list.add(objective ? new ObjectiveQuestionDTO() : new SubjectiveQuestionDTO());
            new ExportExcel(title, objective ? ObjectiveQuestionDTO.class : SubjectiveQuestionDTO.class, 2)
                    .setDataList(list).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            addMessage(redirectAttributes, "导入模板下载失败！失败信息：" + e.getMessage());
        }
        return "redirect:/admin/exam/paper";
    }

    @Logging(menu = "导出客/主观题", type = LogType.EXPORT)
    @RequestMapping(value = "/export")
    public String exportFile(HttpServletRequest request, HttpServletResponse response, @RequestParam Boolean objective,
            RedirectAttributes redirectAttributes) {
        int examId = getSessionExamId(request);

        Map<String, ExamSubject> subjectMap = new HashMap<String, ExamSubject>();
        List<ExamSubject> subjectList = subjectService.list(examId);
        for (ExamSubject subject : subjectList) {
            subjectMap.put(subject.getCode(), subject);
        }

        Map<String, MarkGroup> groupMap = new HashMap<String, MarkGroup>();
        List<MarkGroup> groupList = groupService.findByExam(examId);
        for (MarkGroup group : groupList) {
            groupMap.put(group.getSubjectCode() + "_" + group.getNumber(), group);
        }

        List<QuestionDTO> list = new LinkedList<QuestionDTO>();
        ExamQuestionSearchQuery query = new ExamQuestionSearchQuery();
        query.setExamId(examId);
        query.setObjective(objective);
        query.setSort(new Sort(Direction.ASC, "subjectCode", "paperType", "mainNumber", "subNumber"));
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        query = questionService.findByQuery(query);
        for (ExamQuestion q : query.getResult()) {
            list.add(objective ? new ObjectiveQuestionDTO(q, subjectMap.get(q.getSubjectCode()))
                    : new SubjectiveQuestionDTO(q, subjectMap.get(q.getSubjectCode()), groupMap.get(q.getSubjectCode()
                            + "_" + q.getGroupNumber())));
        }
        try {
            String fileName = objective ? "客观题数据.xlsx" : "主观题数据.xlsx";
            String title = objective ? "客观题数据" : "主观题数据";
            new ExportExcel(title, objective ? ObjectiveQuestionDTO.class : SubjectiveQuestionDTO.class, 2)
                    .setDataList(list).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            addMessage(redirectAttributes, "导出数据失败！失败信息：" + e.getMessage());
        }
        return "redirect:/admin/exam/paper";
    }

    @Logging(menu = "导入客/主观题", type = LogType.IMPORT_FILE)
    @RequestMapping(value = "/import", method = RequestMethod.POST)
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String importFile(HttpServletRequest request, MultipartFile file, @RequestParam Boolean objective,
            RedirectAttributes redirectAttributes) {
        int examId = getSessionExamId(request);

        List<String> error = new LinkedList<String>();
        Map<String, SubjectQuestionDTO> map = parseQuestion(file, examId, objective, error);
        if (error.isEmpty()) {
            int success = 0;
            for (SubjectQuestionDTO dto : map.values()) {
                // 每次导入都需要重新统分
                examService.updateObjectiveStatus(examId, ObjectiveStatus.WAITING);
                ExamSubject subject = subjectService.find(examId, dto.getSubjectCode());
                if (subject != null) {
                    Map<Integer, String> titleMap = new HashMap<>();
                    List<ExamQuestion> current = questionService.findByExamAndSubjectAndObjective(examId,
                            subject.getCode(), objective);
                    for (ExamQuestion question : current) {
                        titleMap.put(question.getMainNumber(), question.getMainTitle());
                    }
                    if (dto.validate(error, titleMap)) {
                        if (objective) {
                            questionService.deleteByExamAndSubjectAndObjective(examId, dto.getSubjectCode(), objective);
                            for (ExamQuestion question : dto.getQuestionList()) {
                                questionService.save(question);
                            }
                            subjectService.updateScore(examId, subject.getCode(), objective, dto.getTotalScore());
                        } else {
                            int questionCount = 0;
                            for (ExamQuestion question : dto.getQuestionList()) {
                                ExamQuestion old = questionService
                                        .findByExamAndSubjectAndObjectiveAndMainNumberAndSubNumber(examId,
                                                subject.getCode(), objective, question.getMainNumber(),
                                                question.getSubNumber());
                                if (old != null) {
                                    if (old.getGroupNumber() != null) {
                                        error.add("[" + subject.getCode() + "] 大题号" + question.getMainNumber() + " 小题号"
                                                + question.getSubNumber() + "已有分组");
                                        continue;
                                    }
                                    old.setTotalScore(question.getTotalScore());
                                    old.setIntervalScore(question.getIntervalScore());
                                    questionService.save(old);
                                } else {
                                    question.setGroupNumber(null);
                                    questionService.save(question);
                                    questionCount++;
                                }
                            }
                            if (questionCount > 0) {
                                studentService.updateSubjectiveStatusAndScoreAndInspectorId(examId, subject.getCode(),
                                        SubjectiveStatus.UNMARK, 0, null, null, null);
                            }
                            subjectService.updateScore(examId, subject.getCode(), objective,
                                    questionService.sumTotalScore(examId, subject.getCode(), objective));
                        }
                        success++;
                    }
                } else {
                    error.add("[" + dto.getSubjectCode() + "] 科目代码不存在；");
                }
            }
            error.add(0, success + "个科目导入成功；");
            if (success > 0) {
                subjectService.updateTotalScore(examId);
            }
            RequestUtils.setLog(request, success + "个科目导入成功；");
        }
        if (error.size() > 0) {
            error.add("<br\\> 导入完毕后请重新统分！");
            addMessage(redirectAttributes, StringUtils.join(error, "<br\\>"));
        }
        return "redirect:/admin/exam/paper";
    }

    @Logging(menu = "导入主观题分组", type = LogType.IMPORT_FILE)
    @RequestMapping(value = "/importGroup", method = RequestMethod.POST)
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String importGroupFile(HttpServletRequest request, MultipartFile file, RedirectAttributes redirectAttributes) {
        int examId = getSessionExamId(request);
        List<String> error = new LinkedList<String>();
        Map<String, SubjectQuestionDTO> map = parseQuestion(file, examId, false, error);

        if (error.isEmpty()) {
            int success = 0;
            for (SubjectQuestionDTO dto : map.values()) {
                // 每次导入都需要重新统分
                examService.updateObjectiveStatus(examId, ObjectiveStatus.WAITING);
                ExamSubject subject = subjectService.find(examId, dto.getSubjectCode());
                if (subject != null) {
                    // 验证是否存在分组
                    List<MarkGroup> groups = groupService.findByExamAndSubject(examId, subject.getCode());
                    if (groups != null && groups.size() > 0) {
                        error.add("[" + subject.getCode() + "] 已存在分组");
                        continue;
                    }
                    Map<Integer, String> titleMap = new HashMap<>();
                    List<ExamQuestion> current = questionService.findByExamAndSubjectAndObjective(examId,
                            subject.getCode(), false);
                    for (ExamQuestion question : current) {
                        titleMap.put(question.getMainNumber(), question.getMainTitle());
                    }
                    if (dto.validate(error, titleMap) && dto.validateGroupNumber(error)
                            && dto.getQuestionList().size() == current.size()) {
                        int successGroup = 0;
                        for (MarkGroup group : dto.getGroups().values()) {
                            if (group.getImportQuestionList() != null) {
                                boolean validate = true;
                                List<ExamQuestion> questionGroup = new ArrayList<ExamQuestion>();
                                for (ExamQuestion question : group.getImportQuestionList()) {
                                    ExamQuestion old = questionService
                                            .findByExamAndSubjectAndObjectiveAndMainNumberAndSubNumber(examId,
                                                    subject.getCode(), false, question.getMainNumber(),
                                                    question.getSubNumber());
                                    if (old == null) {
                                        error.add("[" + group.getSubjectCode() + "_" + group.getNumber() + "] 分组有小题不存在");
                                        validate = false;
                                        break;
                                    }
                                    old.setGroupNumber(group.getNumber());
                                    questionGroup.add(old);
                                }
                                // 校验通过小题才保存
                                if (validate && !questionGroup.isEmpty()) {
                                    questionService.save(questionGroup);
                                }
                                // 有题目的分组才保存
                                if (questionService.countByExamAndSubjectAndObjectiveAndGroupNumber(examId,
                                        group.getSubjectCode(), false, group.getNumber()) != 0) {
                                    double totalScore = 0d;
                                    for (ExamQuestion q : questionGroup) {
                                        totalScore = totalScore + q.getTotalScore();
                                    }
                                    group.setTotalScore(totalScore);
                                    groupService.save(group);
                                    studentService.updateSubjectiveStatusAndScoreAndInspectorId(examId,
                                            group.getSubjectCode(), SubjectiveStatus.UNMARK, 0, null, null, null);
                                    successGroup++;
                                }
                            }
                        }
                        if (successGroup > 0) {
                            success++;
                            subjectService.updateTrialCount(examId, subject.getCode(), dto.getTrialCount());
                        }
                    }
                } else {
                    error.add("[" + dto.getSubjectCode() + "] 科目代码不存在；");
                }
            }
            error.add(0, success + "个科目导入成功；");
            RequestUtils.setLog(request, success + "个科目导入成功；");
        }
        if (error.size() > 0) {
            error.add("<br\\> 导入完毕后请重新统分！");
            addMessage(redirectAttributes, StringUtils.join(error, "<br\\>"));
        }
        return "redirect:/admin/exam/paper";
    }

    private Map<String, SubjectQuestionDTO> parseQuestion(MultipartFile file, int examId, boolean objective,
            List<String> error) {
        Map<String, SubjectQuestionDTO> map = new HashMap<String, SubjectQuestionDTO>();
        try {
            ImportExcel excel = new ImportExcel(file, 1, 0);
            Class<? extends QuestionDTO> clazz = objective ? ObjectiveQuestionDTO.class : SubjectiveQuestionDTO.class;
            List<? extends QuestionDTO> list = excel.getDataList(clazz);
            if (list != null) {
                for (QuestionDTO dto : list) {
                    String subjectCode = StringUtils.trimToNull(dto.getSubjectCode());
                    if (subjectCode != null) {
                        SubjectQuestionDTO subject = map.get(subjectCode);
                        if (subject == null) {
                            subject = new SubjectQuestionDTO(examId, subjectCode, objective);
                            map.put(subjectCode, subject);
                        }
                        subject.addQuestion(dto.transform());
                    }
                }
            }
        } catch (Exception e) {
            log.error("parse question excel error", e);
            error.add("Excel文件解析失败");
        }
        return map;
    }

    @RequestMapping("/getPaperType")
    @ResponseBody
    public JSONArray query(HttpServletRequest request, @RequestParam String subjectCode) {
        int examId = getSessionExamId(request);
        JSONArray array = new JSONArray();
        List<String> list = this.questionService.getPaperType(examId, subjectCode);
        for (String paperType : list) {
            JSONObject obj = new JSONObject();
            obj.accumulate("paperType", paperType);
            array.add(obj);
        }
        return array;
    }

    @RequestMapping(value = "/question-edit/{questionId}", method = RequestMethod.GET)
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String edit(Model model, @PathVariable Integer questionId) {
        ExamQuestion examQuestion = questionService.findById(questionId);
        model.addAttribute("examQuestion", examQuestion);
        model.addAttribute("objectivePolicyList", ObjectivePolicy.values());
        return "modules/exam/questionEdit";
    }

    @Logging(menu = "编辑题目判分策略", type = LogType.UPDATE)
    @RequestMapping(value = "/question-edit", method = RequestMethod.POST)
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String update(@RequestParam Integer id, @RequestParam ObjectivePolicy objectivePolicy) {
        ExamQuestion question = questionService.updateObjectivePolicy(id, objectivePolicy);
        return "redirect:/admin/exam/paper/detail?subjectCode=" + question.getSubjectCode();
    }

    @Logging(menu = "删除题目", type = LogType.DELETE)
    @RequestMapping(value = "/question-delete/{questionId}", method = RequestMethod.GET)
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String delete(HttpServletRequest request, RedirectAttributes redirectAttributes,
            @PathVariable Integer questionId) {
        ExamQuestion question = questionService.findById(questionId);
        String subjectCode = question.getSubjectCode();
        if (markService.deleteByQuestion(question)) {
            addMessage(redirectAttributes, "删除成功");
        } else {
            addMessage(redirectAttributes, "删除失败，该题目已有分组或选做题存在选做题分组");
        }
        return "redirect:/admin/exam/paper/detail?subjectCode=" + subjectCode;
    }

    @Logging(menu = "客观题统分", type = LogType.UPDATE)
    @RequestMapping("/calculate")
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER })
    public ModelAndView calculate(HttpServletRequest request, RedirectAttributes redirectAttributes,
            @RequestParam(required = false) String subjectCode) {
        WebUser wu = RequestUtils.getWebUser(request);
        Exam exam = examService.findById(getSessionExamId(request));
        if (exam == null || !exam.getSchoolId().equals(wu.getUser().getSchoolId())) {
            addMessage(redirectAttributes, "请选择正确的考试");
        } else {
            Set<String> subjectSet = new HashSet<String>();
            // 整个考试加锁
            boolean examLock = false;
            if (subjectCode == null) {
                subjectSet = getCalculateSubjectCodes(exam.getId());
                examLock = true;
            } else if (!lockService.isLocked(LockType.SCORE_CALCULATE, exam.getId(), subjectCode)) {
                subjectSet.add(subjectCode);
            }
            if (!subjectSet.isEmpty()) {
                ScoreCalculateThread thread = new ScoreCalculateThread(exam.getId(), subjectSet, lockService,
                        studentService, questionService, examService, subjectService, fileService, examLock);
                taskExecutor.submit(thread);
            }
        }
        return new ModelAndView("redirect:/admin/exam/paper");
    }

    private Set<String> getCalculateSubjectCodes(Integer examId) {
        List<ExamSubject> list = subjectService.list(examId);
        Set<String> set = new HashSet<>();
        for (ExamSubject subject : list) {
            if (!lockService.isLocked(LockType.SCORE_CALCULATE, subject.getExamId(), subject.getCode())) {
                set.add(subject.getCode());
            }
        }
        return set;
    }

    @Logging(menu = "科目成绩分析计算", type = LogType.UPDATE)
    @RequestMapping("/report")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public ModelAndView report(HttpServletRequest request, RedirectAttributes redirectAttributes,
            @RequestParam(required = false) String subjectCode) {
        WebUser wu = RequestUtils.getWebUser(request);
        Exam exam = examService.findById(getSessionExamId(request));
        if (exam == null || !exam.getSchoolId().equals(wu.getUser().getSchoolId())) {
            addMessage(redirectAttributes, "请选择正确的考试");
        } else {
            Set<String> subjectSet = new HashSet<String>();
            // 整个考试加锁
            boolean examLock = false;
            if (subjectCode == null) {
                examLock = true;
                List<ExamSubject> list = subjectService.list(exam.getId());
                for (ExamSubject subject : list) {
                    if (!lockService.isLocked(LockType.SCORE_CALCULATE, subject.getExamId(), subject.getCode())) {
                        subjectSet.add(subject.getCode());
                    }
                }
            } else if (!lockService.isLocked(LockType.SCORE_CALCULATE, exam.getId(), subjectCode)) {
                subjectSet.add(subjectCode);
            }
            if (!subjectSet.isEmpty()) {
                ScoreReportThread thread = new ScoreReportThread(exam.getId(), subjectSet, lockService, studentService,
                        questionService, reportService, examService, subjectService, examLock);
                taskExecutor.submit(thread);
            }
        }
        return new ModelAndView("redirect:/admin/exam/paper");
    }

    @RequestMapping("/question/query")
    @ResponseBody
    public JSONArray questionQuery(HttpServletRequest request, @RequestParam String subjectCode) {
        int examId = getSessionExamId(request);
        JSONArray array = new JSONArray();
        List<ExamQuestion> list = questionService.findMainByExamAndSubjectAndObjective(examId, subjectCode, false);
        for (ExamQuestion question : list) {
            JSONObject obj = new JSONObject();
            obj.accumulate("mainNumber", question.getMainNumber());
            obj.accumulate("mainTitle", question.getMainTitle());
            array.add(obj);
        }
        return array;
    }
}
