package cn.com.qmth.stmms.admin.exam;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.SelectiveGroup;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.SelectiveGroupService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.enums.LogType;

@Controller("selectiveGroupController")
@RequestMapping("/admin/exam/selectiveGroup")
public class SelectiveGroupController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(SelectiveGroupController.class);

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private SelectiveGroupService selectiveGroupService;

    @Logging(menu = "选做题查询", type = LogType.QUERY)
    @RequestMapping
    public String list(HttpServletRequest request, Model model, @RequestParam String subjectCode) {
        int examId = getSessionExamId(request);
        ExamSubject subject = subjectService.find(examId, subjectCode);
        List<ExamQuestion> questions = questionService.findMainByExamAndSubjectAndObjective(examId, subjectCode, false);
        List<SelectiveGroup> list = selectiveGroupService.findByExamIdAndSubjectCode(examId, subjectCode);
        Set<Integer> mainNumbers = new HashSet<Integer>();
        for (SelectiveGroup selectiveGroup : list) {
            mainNumbers.add(selectiveGroup.getMainNumber());
        }
        for (ExamQuestion examQuestion : questions) {
            if (mainNumbers.contains(examQuestion.getMainNumber())) {
                examQuestion.setSelective(true);
            }
        }
        if (!list.isEmpty()) {
            model.addAttribute("selectiveCount", list.get(0).getSelectiveCount());
        }
        model.addAttribute("subject", subject);
        model.addAttribute("list", list);
        model.addAttribute("questions", questions);
        return "modules/exam/selectiveList";
    }

    @Logging(menu = "选做题设置", type = LogType.ADD)
    @RequestMapping("/add")
    @Transactional
    public String add(HttpServletRequest request, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode, @RequestParam Integer[] mainNumbers, @RequestParam Integer selectiveCount) {
        int examId = getSessionExamId(request);
        if (selectiveCount >= mainNumbers.length) {
            addMessage(redirectAttributes, "选做题数量不能大于已选择题目数量");
            return "redirect:/admin/exam/paper";
        }
        List<MarkGroup> groups = groupService.findByExamAndSubject(examId, subjectCode);
        if (groups != null && groups.size() > 0) {
            addMessage(redirectAttributes, "该科目已经存在分组，无法设置");
            return "redirect:/admin/exam/paper";
        }
        double questionScore = 0;
        if (!checkTotalScore(examId, subjectCode, mainNumbers, questionScore)) {
            addMessage(redirectAttributes, "选做题分数必须一样");
            return "redirect:/admin/exam/paper";
        }
        selectiveGroupService.deleteByExamIdAndSubjectCode(examId, subjectCode);
        if (mainNumbers.length > 0) {
            List<SelectiveGroup> list = new ArrayList<SelectiveGroup>();
            for (Integer mainNumber : mainNumbers) {
                list.add(new SelectiveGroup(examId, subjectCode, mainNumber, 1, selectiveCount));
            }
            selectiveGroupService.save(list);
            Double totalScore = questionService.sumTotalScore(examId, subjectCode, false);
            totalScore = totalScore - (mainNumbers.length - selectiveCount) * questionScore;
            subjectService.updateScore(examId, subjectCode, false, totalScore);
        }
        return "redirect:/admin/exam/paper";
    }

    private boolean checkTotalScore(int examId, String subjectCode, Integer[] mainNumbers, double questionScore) {
        List<ExamQuestion> questions = questionService.findMainByExamAndSubjectAndObjective(examId, subjectCode, false);
        Map<Integer, ExamQuestion> map = new HashMap<Integer, ExamQuestion>();
        for (ExamQuestion examQuestion : questions) {
            map.put(examQuestion.getMainNumber(), examQuestion);
        }
        Double totalScore = null;
        for (Integer mainNumber : mainNumbers) {
            if (totalScore == null) {
                totalScore = map.get(mainNumber).getTotalScore();
            }
            if (totalScore.doubleValue() != map.get(mainNumber).getTotalScore().doubleValue()) {
                return false;
            }
        }
        questionScore = totalScore;
        return true;
    }

    @Logging(menu = "清空选做题设置", type = LogType.ADD)
    @RequestMapping("/clear")
    @Transactional
    public String claer(HttpServletRequest request, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode) {
        int examId = getSessionExamId(request);
        List<MarkGroup> groups = groupService.findByExamAndSubject(examId, subjectCode);
        if (groups != null && groups.size() > 0) {
            addMessage(redirectAttributes, "该科目已经存在分组，无法设置");
            return "redirect:/admin/exam/paper";
        }
        selectiveGroupService.deleteByExamIdAndSubjectCode(examId, subjectCode);
        subjectService.updateScore(examId, subjectCode, false,
                questionService.sumTotalScore(examId, subjectCode, false));
        return "redirect:/admin/exam/paper";
    }
}
